package com.bytesw.bytebot.etl.batch.service;

import com.bytesw.bytebot.etl.batch.beans.DeleteDataSensBean;
import com.bytesw.bytebot.etl.batch.beans.DeleteDataSensControlBean;
import com.bytesw.bytebot.etl.batch.beans.DynaBean;
import com.bytesw.bytebot.etl.batch.beans.TenantBatchBean;
import com.bytesw.bytebot.etl.batch.factory.mapper.RowMapperFactory;
import com.bytesw.bytebot.etl.batch.factory.reader.DataBaseItemReaderFactory;
import com.bytesw.bytebot.etl.batch.processor.ConvertToBeanProcessor;
import com.bytesw.bytebot.etl.batch.writer.ByteBotJPAWriter;
import com.bytesw.bytebot.etl.batch.writer.DataSensibleJPAWriter;
import com.bytesw.bytebot.etl.config.BatchProperties;
import com.bytesw.bytebot.etl.dao.DeleteDataSensibleControlRepository;
import com.bytesw.bytebot.etl.dao.DeleteDataSensibleLogRepository;
import com.bytesw.bytebot.etl.dao.EventHistoryRepository;
import com.bytesw.bytebot.etl.dao.IntentRepository;
import com.bytesw.bytebot.etl.enums.IntentTypeEnum;
import com.bytesw.bytebot.etl.services.DeleteSensMessageService;
import com.bytesw.bytebot.etl.services.ProcessMessageService;
import com.bytesw.bytebot.jdbc.DeleteDataSensJDBCRepository;
import com.bytesw.bytebot.model.enums.AgentParameterEnum;
import com.bytesw.bytebot.model.enums.AgentStatusEnum;
import com.bytesw.bytebot.repository.AgentRepository;
import com.bytesw.xdf.multitenant.core.ThreadLocalStorage;
import com.google.gson.Gson;
import lombok.extern.log4j.Log4j2;
import org.springframework.batch.core.*;
import org.springframework.batch.core.configuration.annotation.JobBuilderFactory;
import org.springframework.batch.core.configuration.annotation.StepBuilderFactory;
import org.springframework.batch.core.explore.JobExplorer;
import org.springframework.batch.core.launch.JobLauncher;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.item.ItemReader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.TaskScheduler;
import org.springframework.scheduling.Trigger;
import org.springframework.scheduling.annotation.SchedulingConfigurer;
import org.springframework.scheduling.config.ScheduledTaskRegistrar;
import org.springframework.scheduling.support.CronTrigger;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.ScheduledFuture;

// Descomentar lo siguiente para que funcione
@Service
@Log4j2
public class ScheduleService implements SchedulingConfigurer {

    ScheduledTaskRegistrar scheduledTaskRegistrar;

    Map<String, ScheduledFuture> futureMap = new HashMap<>();

    @Autowired
    @Qualifier("schedulerLockTaskScheduler")
    TaskScheduler poolScheduler;

    @Value("${application.byte-bot.batch.cron: 0 0/1 * * * * }")
    private String cronExpression;

    @Value("${application.byte-bot.batch.chunk:500}")
    private int chunk;

    @Autowired
    private JobExplorer jobExplorer;

    @Autowired
    @Qualifier("asyncJobLauncher")
    JobLauncher asyncJobLauncher;

    @Autowired
    public JobBuilderFactory jobBuilderFactory;

    @Autowired
    public StepBuilderFactory stepBuilderFactory;

    @Autowired
    private DataBaseItemReaderFactory dataBaseItemReaderFactory;

    @Autowired
    private RowMapperFactory rowMapperFactory;

    @Autowired
    private JobExecutionListener listener;

    @Autowired
    private ProcessMessageService service;

    @Autowired
    private DeleteSensMessageService deleteSensMessageService;

    @Autowired
    private Gson gson;

    @Autowired
    private BatchProperties properties;

    @Autowired
    private EventHistoryRepository eventHistoryRepository;

    @Autowired
    private AgentRepository agentRepository;

    @Autowired
    private DeleteDataSensJDBCRepository deleteDataSensJDBCRepository;

    @Autowired
    private DeleteDataSensibleControlRepository deleteDataSensibleControlRepository;

    @Autowired
    private DeleteDataSensibleLogRepository deleteDataSensibleLogRepository;

    @Autowired
    private IntentRepository intentRepository;

    @Override
    public void configureTasks(ScheduledTaskRegistrar taskRegistrar) {
        if (scheduledTaskRegistrar == null) {
            scheduledTaskRegistrar = taskRegistrar;
        }

        if (taskRegistrar.getScheduler() == null) {
            taskRegistrar.setScheduler(poolScheduler);
        }

        if (properties == null) {
            throw new RuntimeException("batch properties not found");
        }

        if (properties.getTenants() == null) {
            throw new RuntimeException("batch properties not found");
        }

        // identifier "prueba" Cambiar luego

        properties.getTenants().stream().forEach(tenant -> {
            ThreadLocalStorage.setTenantName(tenant.getId());
            configureTask(tenant.getId(), "prueba", taskRegistrar);
        });

        //@TODO Falta añadir el modo multitenant
    }

    private void configureTask(String tenantIdentifier, String identifier, ScheduledTaskRegistrar taskRegistrar) {
        Trigger trigger = new CronTrigger(cronExpression, TimeZone.getDefault());
        /* ETL de Dashboard */
        String key = String.format("%s-%s", tenantIdentifier, identifier);
        futureMap.put(key, taskRegistrar.getScheduler().schedule(() -> scheduleCron(createJob(tenantIdentifier), tenantIdentifier), trigger));

        /*ETL eliminacion de data sensible*/
        List<DeleteDataSensBean> deleteDataSensBeans = deleteDataSensJDBCRepository
                .getListAgentChannel(AgentStatusEnum.DEPLOYED.getName(), AgentParameterEnum.ACCESS_TWILIO.getName());
        String keyDataSens = "";
        for (DeleteDataSensBean data : deleteDataSensBeans) {
            keyDataSens = String.format("%s-%s", tenantIdentifier, data.getValue());
            futureMap.put(keyDataSens, taskRegistrar.getScheduler()
                    .schedule(() -> scheduleCron(createJobDataSens(tenantIdentifier, data), tenantIdentifier), trigger));
        }
    }

    /* Métodos utilizados para ETL de dashboard */

    private Job createJob(String tenantIdentifier) {
        ThreadLocalStorage.setTenantName(tenantIdentifier);
        return jobBuilderFactory.get("processJob")
                .incrementer(new RunIdIncrementer()).listener(listener)
                .flow(createStep(tenantIdentifier)).end().build();
    }

    private Step createStep(String tenantIdentifier) {
        ByteBotJPAWriter writer = new ByteBotJPAWriter();
        writer.setService(service);

        return stepBuilderFactory.get("processByteBotData").<DynaBean, DynaBean>chunk(chunk)
                .reader(getReader(tenantIdentifier))
                .processor(new ConvertToBeanProcessor())
                .writer(writer).build();
    }

    private ItemReader getReader(String tenantIdentifier) {
        if (properties == null) {
            throw new RuntimeException("Properties not found");
        }

        Optional<TenantBatchBean> tenantFound = findTenant(tenantIdentifier);
        if (!tenantFound.isPresent()) {
            throw new RuntimeException("Properties not found");
        }

        Long id = eventHistoryRepository.maxEventId();
        Map<String, Object> params = new HashMap<>();
        params.putAll(tenantFound.get().getDatasource());
        params.put("query", String.format((String) params.get("query"), id != null ? id : 0));

        return dataBaseItemReaderFactory.createReader(rowMapperFactory.createMapper(new HashMap<>()), params);
    }

    /* Métodos ETL de data sensible */

    private Job createJobDataSens(String tenantIdentifier, DeleteDataSensBean data) {
        ThreadLocalStorage.setTenantName(tenantIdentifier);
        return jobBuilderFactory.get("processJobDataSensible")
                .incrementer(new RunIdIncrementer()).listener(listener)
                .flow(createStepDataSens(tenantIdentifier, data)).end().build();
    }

    private Step createStepDataSens(String tenantIdentifier, DeleteDataSensBean data) {
        DataSensibleJPAWriter writer = new DataSensibleJPAWriter();
        List<String> intentByAgent = deleteDataSensJDBCRepository.getIntentByAgent(IntentTypeEnum.SENSIBLE.getName(), data.getAgenId());
        writer.setIntentByAgent(intentByAgent);
        writer.setAgent(data);
        writer.setService(deleteSensMessageService);
        writer.setIntentRepository(intentRepository);
        writer.setDeleteDataSensibleControlRepository(deleteDataSensibleControlRepository);
        writer.setDeleteDataSensibleLogRepository(deleteDataSensibleLogRepository);

        return stepBuilderFactory.get("processByteBotDatasSens")
                .<DynaBean, DynaBean>chunk(chunk)
                .reader(getReaderDataSens(tenantIdentifier, data))
                .processor(new ConvertToBeanProcessor())
                .writer(writer).build();
    }

    private ItemReader getReaderDataSens(String tenantIdentifier, DeleteDataSensBean data) {
        if (properties == null) {
            throw new RuntimeException("Properties not found");
        }

        Optional<TenantBatchBean> tenantFound = findTenant(tenantIdentifier);
        if (!tenantFound.isPresent()) {
            throw new RuntimeException("Properties not found");
        }

        DeleteDataSensControlBean control = deleteDataSensJDBCRepository.getControl(data.getAgenId());
        OffsetDateTime endDate = OffsetDateTime.now();
        Long id = 0L;
        int differenceTime = 0;

        if (control != null) {
            OffsetDateTime lastSessionDate = OffsetDateTime.ofInstant(control.getDateDelete().toInstant(), ZoneId.systemDefault());
            Duration difference = Duration.between(lastSessionDate, endDate);

            differenceTime = (int) (difference.getSeconds() / 60);

            if (differenceTime < data.getDeletePeriod()) {
                log.info("Agente " + data.getValue() + " aun no cumple con el periodo de espera.");
                id = null;
            } else {
                id = control.getEventId();
            }
        }

        Map<String, Object> params = new HashMap<>();
        params.putAll(tenantFound.get().getDatasource());
        params.put("query", String.format((String) params.get("query"), id));

        return dataBaseItemReaderFactory.createReader(rowMapperFactory.createMapper(new HashMap<>()), params);
    }

    /* Métodos generales */

    private Optional<TenantBatchBean> findTenant(String tenantId) {
        return properties.getTenants().stream().filter(x -> x.getId().equals(tenantId)).findFirst();
    }

    private void scheduleCron(Job job, String tenantId) {
        UUID traceID = UUID.randomUUID();
        Map<String, JobParameter> maps = new HashMap<>();
        maps.put("time", new JobParameter(System.currentTimeMillis()));
        maps.put("uuid", new JobParameter(traceID.toString()));
        maps.put("tenantId", new JobParameter(tenantId));
        JobParameters parameters = new JobParameters(maps);
        Set<JobExecution> jobExecutions = new HashSet<>();
        try {
            jobExecutions = jobExplorer.findRunningJobExecutions(job.getName());
            String JobInfo = String.format("Jobs {} en Ejecución: {}", job.getName(), jobExecutions.size());
            log.info(JobInfo);
            if (jobExecutions.isEmpty()) {
                asyncJobLauncher.run(job, parameters);
            } else {
                log.info("El Job " + job.getName() + " no se ejecutará porque hay jobs pendientes: " + jobExecutions.size());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
