package com.bytesw.bytebot.model;

import com.bytesw.bytebot.model.converters.FrequentQuestionStatusConverter;
import com.bytesw.bytebot.model.converters.StatusConverter;
import com.bytesw.bytebot.model.enums.FrequentQuestionStatusEnum;
import com.bytesw.bytebot.model.enums.StatusEnum;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;

import javax.persistence.*;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;

/**
 * @author Sebastian Chicoma Sandmann
 * @version 09-sep-2020
 *
 * Copyright (c) 2020 Byte, S.A. Todos los derechos reservados.
 *
 * Este software constituye información confidencial y propietaria de Byte, S.A.
 * ("Información Confidencial"). Usted no debe develar dicha Información
 * Confidencial y debe usarla de acuerdo con los términos de aceptación de
 * licencia de uso que firmó con Byte.
 */
@Entity
@Table(name = "AVB_FREQUENT_QUESTION")
@Getter
@Setter
@ToString
@EqualsAndHashCode(of = {"id"}, callSuper = false)
public class FrequentQuestion {
    @Id
    @Column(name = "FQUE_ID")
    @GeneratedValue(strategy = GenerationType.TABLE, generator = "AVB_FREQUENT_QUESTION_GENERATOR")
    @TableGenerator(name = "AVB_FREQUENT_QUESTION_GENERATOR", table = "SEQUENCE_TABLE", pkColumnName = "SEQ_NAME", valueColumnName = "SEQ_COUNT",
            pkColumnValue = "AVB_FREQUENT_QUESTION_SEQ", initialValue = 1, allocationSize = 1)
    private Long id;

    @Column(name = "FQUE_UUID")
    private String uuid;

    @Column(name = "FQUE_NAME")
    private String filename;

    @Column(name = "FQUE_DESC")
    private String description;

    @Column(name = "FQUE_STATE")
    @Convert(converter = FrequentQuestionStatusConverter.class)
    private FrequentQuestionStatusEnum status;

    @ManyToOne
    @JoinColumn(name = "AGEN_ID", referencedColumnName = "AGEN_ID")
    private Agent agent;

    @Column(name = "FQUE_USER")
    private String user;

    @Column(name = "FQUE_LDATE", columnDefinition = "TIMESTAMP")
    private LocalDateTime uploadDate;

}
